import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.apache.hadoop.io.DoubleWritable;
import org.apache.hadoop.io.LongWritable;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.mrunit.mapreduce.MapDriver;
import org.apache.hadoop.mrunit.mapreduce.ReduceDriver;

/**
 * This class contains the public test cases for the project. The public tests
 * are distributed with the project description, and students can run the public
 * tests themselves against their own implementation.
 * 
 * Any changes to this file will be ignored when testing your project.
 * 
 */
public class PublicTests extends BaseTests {
	static {
		input="data/inputDirectory";
		try {
			MatrixVectorMult.launchJob(input, intermediary, output);
			readOutputVector();
		} catch (IOException | ClassNotFoundException | InterruptedException e) {
			initializationError = e.toString();
		}
	}
	
	public void testFirstMapper() throws IOException{
		MatrixVectorMult.FirstMap m = new MatrixVectorMult.FirstMap();
		MapDriver<LongWritable, Text, Text, Text> mapDriver = MapDriver.newMapDriver(m);
		mapDriver.withInput(new LongWritable(0),new Text("1 1 0"));
		mapDriver.withOutput(new Text("1"), new Text("1 0"));
		mapDriver.runTest();
	}
	
	public void testSecondMapper() throws IOException{
		MatrixVectorMult.SecondMap m = new MatrixVectorMult.SecondMap();
		MapDriver<Text, Text, Text, Text> mapDriver = MapDriver.newMapDriver(m);
		mapDriver.withInput(new Text("1"),new Text("0"));
		mapDriver.withOutput(new Text("1"),new Text("0"));
		mapDriver.runTest();
	}
	
	public void testCombiner() throws IOException {
		MatrixVectorMult.CombinerForSecondMap c = new MatrixVectorMult.CombinerForSecondMap();
		ReduceDriver<Text, Text, Text, Text> combinerDriver = ReduceDriver.newReduceDriver(c);
		List<Text> values = new ArrayList<Text>();
		values.add(new Text("0.5")) ;
		values.add(new Text("0.5"));
		
		combinerDriver.withInput(new Text("1"), values);
		combinerDriver.withOutput(new Text("1"), new Text("1.0"));
		combinerDriver.runTest();
	}
	
	
	public void testSecondReducer() throws IOException {
		MatrixVectorMult.SecondReduce r = new MatrixVectorMult.SecondReduce();
		ReduceDriver<Text, Text, Text, DoubleWritable> reduceDriver = ReduceDriver.newReduceDriver(r);
		List<Text> values = new ArrayList<Text>();
		values.add(new Text("0.25"));
		values.add(new Text("0.25"));
		values.add(new Text("0.25"));
		values.add(new Text("0.25"));
		reduceDriver.withInput(new Text("2"), values);
		reduceDriver.withOutput(new Text("2"), new DoubleWritable(1.0));
		reduceDriver.runTest();
	}

	public void testVectorSize() {
		assertEquals(4, outputVector.size());
	}
	
	public void testVectorSumofElements()
	{
		double sum = 0.;
		for(int line:outputVector.keySet())
		{
			sum += outputVector.get(line);
		}
		assertEquals(1.0, sum, 0.1);
	}
	
}

